
/******************************************************************
  (C) Copyright 2001 by UNITED BYTES
   
  web:  http://www.unitedbytes.de
        http://www.unitedbytes.org
		http://www.unitedbytes.net
  mail: info@unitedbytes.de

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, 
  Boston, MA  02111-1307, USA.

  *****************************************************************

  Program:      ubNetwork
  File:         sources/cub_n_client.cpp
  Author:       Andreas Loeffler
  Date:         Mon, 10.04.2001
  Version:      1.0
  Description:  Client base class for internet applications

  -----------------------------------------------------------------

  Changes:		-
  Date:			-
  Description:	-

  *****************************************************************/

#include "cub_n_client.h"

////////////////////////////////////////////
// States:
//			1:	WinSock init
//			2:  Socket created & binded
//
////////////////////////////////////////////

CUB_Client::CUB_Client ()
{
	// Set init level
	m_iInitLevel = CUB_NS_NOT_INITIALIZED;

	// Clear error text
	memset (m_cErrorText, 0, sizeof(m_cErrorText));
}

CUB_Client::~CUB_Client ()
{
	Shutdown ();
}

bool CUB_Client::Setup (char *cServerIP, int iPort)
{
	if ((iPort == 0) ||
		(!strcmp (cServerIP  , "")))
	{
		sprintf (m_cErrorText, "CUB_Client->Setup: Wrong connect parameters given! Please check them!");
		return false;
	}
	else {
		sprintf (m_cServerIP, "%s", cServerIP);
		m_iServerPort = iPort;
	}

	return true;
}

bool CUB_Client::Initialize ()
{
	WSADATA WSAData;
	int iError = 0;

	if (m_iInitLevel > CUB_NS_NOT_INITIALIZED)
	{
		sprintf (m_cErrorText, "CUB_Client->Initialize: Client already initialized!");
		iError = 1;
	}
	else {
		// Call "WSAStartup", request WSA version 2.2 or 1.1
		iError = WSAStartup(MAKEWORD(2, 2), &WSAData);

		if (iError != 0) {
			sprintf (m_cErrorText, "CUB_Client->Initialize: Could not find a valid WinSock DLL!");
			iError = 1;
		}
		else {
			m_iInitLevel = CUB_NS_WINSOCK_INIT;

			if (!CreateSocket()) {
				iError = 1;
			}
			else {
				sprintf (m_cErrorText, "CUB_Client->Initialize: Success.");
			}
		}
	}

	if (iError == 1) 
		return false;

	return true;	
}

bool CUB_Client::Shutdown ()
{
	int iError = 0;

	if (m_iInitLevel < CUB_NS_WINSOCK_INIT)
	{
		iError = 1;
		sprintf (m_cErrorText, "CUB_Client->Shutdown: Cant shutdown, cause server is not initialized yet!");
	}
	else {
		switch (m_iInitLevel)
		{
			case CUB_NS_CONNECTED:

			case CUB_NS_SOCKET_CREATED:
				m_Socket.CloseSocket ();

			case CUB_NS_WINSOCK_INIT:
				WSACleanup();
				m_iInitLevel = CUB_NS_NOT_INITIALIZED;
				break;

			default:
				sprintf (m_cErrorText, "CUB_Client->Shutdown: Wrong InitLevel detected!");
				iError = 1;
				break;
		}
	}

	memset (m_cServerIP, 0, sizeof(m_cServerIP));
	m_iServerPort = 0;

	if (iError == 1)
		return false;

	sprintf (m_cErrorText, "CUB_Client->Shutdown: Success.");
	return true;
}

bool CUB_Client::CloseConnection ()
{
	int iError = 0;

	if (m_iInitLevel != CUB_NS_CONNECTED)
	{
		sprintf (m_cErrorText, "CUB_Client->CloseConnection: Not connected yet!");
		iError = 1;
	}
	else {
		

		if (!m_Socket.CloseSocket ()) {
			sprintf (m_cErrorText, "CUB_Client->CloseConnection: Cant close socket!");
			iError = 1;
		}
		else {
			m_iInitLevel = CUB_NS_SOCKET_CREATED;
		}
	}

	if (iError == 1)
		return false;

	sprintf (m_cErrorText, "CUB_Client->CloseConnection: Success.");
	return true;
}

bool CUB_Client::Connect ()
{
	int iError = 0;

	if (m_iInitLevel != CUB_NS_SOCKET_CREATED)
	{
		sprintf (m_cErrorText, "CUB_Client->Connect: Wrong InitLevel detected!");
		iError = 1;
	}
	else {
		if (!m_Socket.Connect (m_cServerIP, m_iServerPort)) {
			sprintf (m_cErrorText, "CUB_Client->Connect: Connection attempt to address %s, port %d failed!", m_cServerIP, m_iServerPort);
			iError = 1;
		}
		else {
			// Client successfully connected
			m_iInitLevel = CUB_NS_CONNECTED;
		}
	}

	if (iError == 1)
		return false;

	sprintf (m_cErrorText, "CUB_Client->Connect: Success. (Host=%s, Port=%d)", m_cServerIP, m_iServerPort);
	return true;
}

bool CUB_Client::CreateSocket ()
{
	int iError = 0;

	if (m_iInitLevel != CUB_NS_WINSOCK_INIT)
	{
		sprintf (m_cErrorText, "CUB_Client->CreateSocket: Wrong InitLevel detected!");
		iError = 1;
	}
	else {
		// Create the socket
		if (!m_Socket.CreateSocket()) {
			sprintf (m_cErrorText, "CUB_Client->CreateSocket: Cannot create socket! WinSock-Error: %d", GetLastError());
			iError = 1;	
		}
		else {
			// Set socket to non blocking mode
			// m_Socket.SetSocketBlockingMode (false);

			// Set the init level
			m_iInitLevel = CUB_NS_SOCKET_CREATED;
		}
	}

	if (iError == 1)
		return false;

	sprintf (m_cErrorText, "CUB_Client->CreateSocket: Success.");
	return true;
}

bool CUB_Client::GetLocalIP (char *cBuffer)
{
	int iError = 0;

	if (m_iInitLevel < CUB_NS_WINSOCK_INIT)
	{
		sprintf (m_cErrorText, "CUB_Client->GetLocalIP: Wrong InitLevel detected!");
		iError = 1;
	}
	else {
		if (!m_Socket.GetLocalIP (cBuffer)) {
			sprintf (m_cErrorText, "CUB_Client->GetLocalIP: Could not lookup own IP address!");
			iError = 1;	
		}
	}

	if (iError == 1)
		return false;

	sprintf (m_cErrorText, "CUB_Client->GetLocalIP: (%s) Success.", cBuffer);
	return true;
}

bool CUB_Client::IsConnected ()
{
	if (m_iInitLevel >= CUB_NS_CONNECTED)
		return true;

	return false;
}

